@@ -382,8 +382,8 @@ g_vfs_backend_ftp_finalize (GObject *object)
 
   /* has been cleared on unmount */
   g_assert (ftp->queue == NULL);
-  g_cond_free (ftp->cond);
-  g_mutex_free (ftp->mutex);
+  g_cond_clear (&ftp->cond);
+  g_mutex_clear (&ftp->mutex);
 
   g_free (ftp->user);
   g_free (ftp->password);
@@ -395,8 +395,6 @@ g_vfs_backend_ftp_finalize (GObject *object)
 static void
 g_vfs_backend_ftp_init (GVfsBackendFtp *ftp)
 {
-  ftp->mutex = g_mutex_new ();
-  ftp->cond = g_cond_new ();
 }
 
 static void
@@ -485,10 +483,10 @@ restart:
         {
           if (ftp->has_initial_user)
             /* Translators: the first %s is the username, the second the host name */
-            prompt = g_strdup_printf (_("Enter password for ftp as %s on %s"), ftp->user, ftp->host_display_name);
+            prompt = g_strdup_printf (_("Enter password for FTP as %s on %s"), ftp->user, ftp->host_display_name);
           else
             /* translators: %s here is the hostname */
-            prompt = g_strdup_printf (_("Enter password for ftp on %s"), ftp->host_display_name);
+            prompt = g_strdup_printf (_("Enter password for FTP on %s"), ftp->host_display_name);
         }
          
       flags = G_ASK_PASSWORD_NEED_PASSWORD;
@@ -627,11 +625,11 @@ try_login:
     g_mount_spec_set (mount_spec, "user", ftp->user);
      
   if (g_str_equal (ftp->user, "anonymous"))
-    display_name = g_strdup_printf (_("ftp on %s"), ftp->host_display_name);
+    display_name = g_strdup_printf (_("FTP on %s"), ftp->host_display_name);
   else
     {
       /* Translators: the first %s is the username, the second the host name */
-      display_name = g_strdup_printf (_("ftp as %s on %s"), ftp->user, ftp->host_display_name);
+      display_name = g_strdup_printf (_("FTP as %s on %s"), ftp->user, ftp->host_display_name);
     }
   g_vfs_backend_set_mount_spec (backend, mount_spec);
   g_mount_spec_unref (mount_spec);
@@ -698,7 +696,7 @@ do_unmount (GVfsBackend *   backend,
   GVfsBackendFtp *ftp = G_VFS_BACKEND_FTP (backend);
   GVfsFtpConnection *conn;
 
-  g_mutex_lock (ftp->mutex);
+  g_mutex_lock (&ftp->mutex);
   while ((conn = g_queue_pop_head (ftp->queue)))
     {
       /* FIXME: properly quit */
@@ -706,8 +704,8 @@ do_unmount (GVfsBackend *   backend,
     }
   g_queue_free (ftp->queue);
   ftp->queue = NULL;
-  g_cond_broadcast (ftp->cond);
-  g_mutex_unlock (ftp->mutex);
+  g_cond_broadcast (&ftp->cond);
+  g_mutex_unlock (&ftp->mutex);
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
@@ -1085,7 +1083,7 @@ try_query_settable_attributes (GVfsBackend *backend,
                         G_IO_ERROR,
                         G_IO_ERROR_NOT_SUPPORTED,
                         _("Operation unsupported"));
-      return FALSE;
+      return TRUE;
     }    
 
   list = g_file_attribute_info_list_new ();
