@@ -28,6 +28,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
@@ -41,21 +42,27 @@
 #include "gvfsjobqueryinfo.h"
 #include "gvfsjobenumerate.h"
 #include "gvfsdaemonprotocol.h"
+#include "gvfskeyring.h"
 #include "gmounttracker.h"
 
 #include <libsmbclient.h>
 #include "libsmb-compat.h"
 
-#ifdef HAVE_GCONF
-#include <gconf/gconf-client.h>
-#endif
-
-/* We load a default workgroup from gconf */
-#define PATH_GCONF_GNOME_VFS_SMB_WORKGROUP "/system/smb/workgroup"
-
 /* The magic "default workgroup" hostname */
 #define DEFAULT_WORKGROUP_NAME "X-GNOME-DEFAULT-WORKGROUP"
 
+/* Time in seconds before we mark dirents cache outdated */
+#define DEFAULT_CACHE_EXPIRATION_TIME 10
+
+
+#define PRINT_DEBUG 
+
+#ifdef PRINT_DEBUG 
+#define DEBUG(msg...) g_print("### SMB-BROWSE: " msg)
+#else 
+#define DEBUG(...) 
+#endif 
+
 typedef struct {
   unsigned int smbc_type;
   char *name;
@@ -72,15 +79,28 @@ struct _GVfsBackendSmbBrowse
   char *domain;
   char *server;
   char *mounted_server; /* server or DEFAULT_WORKGROUP_NAME */
+  char *default_workgroup;
   SMBCCTX *smb_context;
 
-  GMutex *entries_lock;
+  char *last_user;
+  char *last_domain;
+  char *last_password;
+
+  GMountSource *mount_source;
+  int mount_try;
+  gboolean mount_try_again;
+  gboolean mount_cancelled;
+  
+  gboolean password_in_keyring;
+  GPasswordSave password_save;
+
+  GMutex entries_lock;
+  GMutex update_cache_lock;
   time_t last_entry_update;
   GList *entries;
   int entry_errno;
 };
 
-static char *default_workgroup = NULL;
 
 static GHashTable *server_cache = NULL;
 static GMountTracker *mount_tracker = NULL;
@@ -204,8 +224,10 @@ g_vfs_backend_smb_browse_finalize (GObject *object)
   g_free (backend->domain);
   g_free (backend->mounted_server);
   g_free (backend->server);
+  g_free (backend->default_workgroup);
   
-  g_mutex_free (backend->entries_lock);
+  g_mutex_clear (&backend->entries_lock);
+  g_mutex_clear (&backend->update_cache_lock);
 
   smbc_free_context (backend->smb_context, TRUE);
   
@@ -219,10 +241,24 @@ g_vfs_backend_smb_browse_finalize (GObject *object)
 static void
 g_vfs_backend_smb_browse_init (GVfsBackendSmbBrowse *backend)
 {
-  backend->entries_lock = g_mutex_new ();
+  char *workgroup;
+  GSettings *settings;
 
   if (mount_tracker == NULL)
     mount_tracker = g_mount_tracker_new (NULL);
+
+  /* Get default workgroup name */
+  settings = g_settings_new ("org.gnome.system.smb");
+
+  workgroup = g_settings_get_string (settings, "workgroup");
+  if (workgroup && workgroup[0])
+    backend->default_workgroup = workgroup;
+  else
+    g_free (workgroup);
+
+  g_object_unref (settings);
+
+  DEBUG ("g_vfs_backend_smb_browse_init: default workgroup = '%s'\n", backend->default_workgroup ? backend->default_workgroup : "NULL");
 }
 
 /**
@@ -256,14 +292,129 @@ auth_callback (SMBCCTX *context,
 	       char *password_out, int pwmaxlen)
 {
   GVfsBackendSmbBrowse *backend;
+  char *ask_password, *ask_user, *ask_domain;
+  gboolean handled, abort;
 
   backend = smbc_getOptionUserData (context);
 
+  strncpy (password_out, "", pwmaxlen);
+
   if (backend->domain)
     strncpy (domain_out, backend->domain, domainmaxlen);
   if (backend->user)
     strncpy (username_out, backend->user, unmaxlen);
-  strncpy (password_out, "", pwmaxlen);
+
+  if (backend->mount_cancelled)
+    {
+      /*  Don't prompt for credentials, let smbclient finish the mount loop  */
+      strncpy (username_out, "ABORT", unmaxlen);
+      strncpy (password_out, "", pwmaxlen);
+      DEBUG ("auth_callback - mount_cancelled\n");
+      return;
+    }
+
+  if (backend->mount_source == NULL)
+    {
+      /* Not during mount, use last password */
+      if (backend->last_user)
+        strncpy (username_out, backend->last_user, unmaxlen);
+      if (backend->last_domain)
+        strncpy (domain_out, backend->last_domain, domainmaxlen);
+      if (backend->last_password)
+        strncpy (password_out, backend->last_password, pwmaxlen);
+      return;
+    }
+
+  if (backend->mount_try == 0 &&
+      backend->user == NULL &&
+      backend->domain == NULL)
+    {
+      /* Try again if kerberos login + anonymous fallback fails */
+      backend->mount_try_again = TRUE;
+      DEBUG ("auth_callback - anonymous pass\n");
+    }
+  else
+    {
+      gboolean in_keyring = FALSE;
+
+      DEBUG ("auth_callback - normal pass\n");
+
+      if (!backend->password_in_keyring)
+        {
+	  in_keyring = g_vfs_keyring_lookup_password (backend->user,
+						      backend->server,
+						      backend->domain,
+						      "smb",
+						      NULL,
+						      NULL,
+						      0,
+						      &ask_user,
+						      &ask_domain,
+						      &ask_password);
+	  backend->password_in_keyring = in_keyring;
+	}
+
+      if (!in_keyring)
+        {
+	  GAskPasswordFlags flags = G_ASK_PASSWORD_NEED_PASSWORD;
+	  char *message;
+
+	  if (g_vfs_keyring_is_available ())
+	    flags |= G_ASK_PASSWORD_SAVING_SUPPORTED;
+	  if (backend->domain == NULL)
+	    flags |= G_ASK_PASSWORD_NEED_DOMAIN;
+	  if (backend->user == NULL)
+	    flags |= G_ASK_PASSWORD_NEED_USERNAME;
+
+	  DEBUG ("auth_callback - asking for password...\n");
+
+	  /* translators: %s is a server name */
+	  message = g_strdup_printf (_("Password required for %s"),
+				     server_name);
+	  handled = g_mount_source_ask_password (backend->mount_source,
+						 message,
+						 username_out,
+						 domain_out,
+						 flags,
+						 &abort,
+						 &ask_password,
+						 &ask_user,
+						 &ask_domain,
+						 NULL,
+						 &(backend->password_save));
+	  g_free (message);
+	  if (!handled)
+	    goto out;
+
+	  if (abort)
+	    {
+	      strncpy (username_out, "ABORT", unmaxlen);
+	      strncpy (password_out, "", pwmaxlen);
+	      backend->mount_cancelled = TRUE;
+	      goto out;
+	    }
+	}
+
+      /* Try again if this fails */
+      backend->mount_try_again = TRUE;
+
+      strncpy (password_out, ask_password, pwmaxlen);
+      if (ask_user && *ask_user)
+	strncpy (username_out, ask_user, unmaxlen);
+      if (ask_domain && *ask_domain)
+	strncpy (domain_out, ask_domain, domainmaxlen);
+
+    out:
+      g_free (ask_password);
+      g_free (ask_user);
+      g_free (ask_domain);
+    }
+
+  backend->last_user = g_strdup (username_out);
+  backend->last_domain = g_strdup (domain_out);
+  backend->last_password = g_strdup (password_out);
+  DEBUG ("auth_callback - out: last_user = '%s', last_domain = '%s'\n", 
+         backend->last_user, backend->last_domain);
 }
 
 /* Add a server to the cache system
@@ -419,8 +570,8 @@ g_string_append_encoded (GString *string,
     }
 }
 
-static void
-update_cache (GVfsBackendSmbBrowse *backend)
+static gboolean
+update_cache (GVfsBackendSmbBrowse *backend, SMBCFILE *supplied_dir)
 {
   GString *uri;
   char dirents[1024*4];
@@ -436,6 +587,11 @@ update_cache (GVfsBackendSmbBrowse *backend)
 
   entries = NULL;
   entry_errno = 0;
+  res = -1;
+
+  g_mutex_lock (&backend->update_cache_lock);
+  
+  DEBUG ("update_cache - updating...\n");
   
   /* Update Cache */
   uri = g_string_new ("smb://");
@@ -450,7 +606,7 @@ update_cache (GVfsBackendSmbBrowse *backend)
   smbc_getdents = smbc_getFunctionGetdents (backend->smb_context);
   smbc_closedir = smbc_getFunctionClosedir (backend->smb_context);
 
-  dir = smbc_opendir (backend->smb_context, uri->str);
+  dir = supplied_dir ? supplied_dir : smbc_opendir (backend->smb_context, uri->str);
   g_string_free (uri, TRUE);
   if (dir == NULL)
     {
@@ -462,7 +618,12 @@ update_cache (GVfsBackendSmbBrowse *backend)
     {
       res = smbc_getdents (backend->smb_context, dir, (struct smbc_dirent *)dirents, sizeof (dirents));
       if (res <= 0)
-	break;
+        {
+          if (res < 0)
+            DEBUG ("update_cache - smbc_getdents returned %d, errno = [%d] %s\n", 
+                   res, errno, g_strerror (errno));
+	  break;
+	}  
       
       dirp = (struct smbc_dirent *)dirents;
       while (res > 0)
@@ -494,13 +655,14 @@ update_cache (GVfsBackendSmbBrowse *backend)
 
       entries = g_list_reverse (entries);
     }
-      
-  smbc_closedir (backend->smb_context, dir);
+
+  if (! supplied_dir)
+    smbc_closedir (backend->smb_context, dir);
 
 
  out:
 
-  g_mutex_lock (backend->entries_lock);
+  g_mutex_lock (&backend->entries_lock);
   
   /* Clear old cache */
   g_list_foreach (backend->entries, (GFunc)browse_entry_free, NULL);
@@ -509,8 +671,12 @@ update_cache (GVfsBackendSmbBrowse *backend)
   backend->entry_errno = entry_errno;
   backend->last_entry_update = time (NULL);
 
-  g_mutex_unlock (backend->entries_lock);
-  
+  DEBUG ("update_cache - done.\n");
+
+  g_mutex_unlock (&backend->entries_lock);
+  g_mutex_unlock (&backend->update_cache_lock);
+
+  return (res >= 0);
 }
 
 static BrowseEntry *
@@ -611,19 +777,26 @@ has_name (GVfsBackendSmbBrowse *backend,
 {
   gboolean res;
   
-  g_mutex_lock (backend->entries_lock);
+  g_mutex_lock (&backend->entries_lock);
   res = (find_entry_unlocked (backend, filename) != NULL);
-  g_mutex_unlock (backend->entries_lock);
+  g_mutex_unlock (&backend->entries_lock);
   return res;
 }
 
 static gboolean
 cache_needs_updating (GVfsBackendSmbBrowse *backend)
 {
-  time_t now = time (NULL);
+  time_t now;
+  gboolean res;
+
+  /*  If there's already cache update in progress, lock and wait until update is finished, then recheck  */
+  g_mutex_lock (&backend->update_cache_lock);
+  now = time (NULL);
+  res = now < backend->last_entry_update ||
+    (now - backend->last_entry_update) > DEFAULT_CACHE_EXPIRATION_TIME;
+  g_mutex_unlock (&backend->update_cache_lock);
   
-  return now < backend->last_entry_update ||
-    (now - backend->last_entry_update) > 10;
+  return res; 
 }
 
 static void
@@ -635,10 +808,17 @@ do_mount (GVfsBackend *backend,
 {
   GVfsBackendSmbBrowse *op_backend = G_VFS_BACKEND_SMB_BROWSE (backend);
   SMBCCTX *smb_context;
+  SMBCFILE *dir;
   char *display_name;
+  const char *debug;
+  int debug_val;
   char *icon;
+  GString *uri;
+  gboolean res;
   GMountSpec *browse_mount_spec;
-
+  smbc_opendir_fn smbc_opendir;
+  smbc_closedir_fn smbc_closedir;
+  
   smb_context = smbc_new_context ();
   if (smb_context == NULL)
     {
@@ -650,7 +830,17 @@ do_mount (GVfsBackend *backend,
 
   smbc_setOptionUserData (smb_context, backend);
 
-  smbc_setDebug (smb_context, 0);
+  debug = g_getenv ("GVFS_SMB_DEBUG");
+  if (debug)
+    debug_val = atoi (debug);
+  else
+#ifdef PRINT_DEBUG
+    debug_val = 4;
+#else
+    debug_val = 0;
+#endif
+
+  smbc_setDebug (smb_context, debug_val);
   smbc_setFunctionAuthDataWithContext (smb_context, auth_callback);
   
   smbc_setFunctionAddCachedServer (smb_context, add_cached_server);
@@ -658,18 +848,20 @@ do_mount (GVfsBackend *backend,
   smbc_setFunctionRemoveCachedServer (smb_context, remove_cached_server);
   smbc_setFunctionPurgeCachedServers (smb_context, purge_cached);
   
-  /* FIXME: is strdup() still needed here? -- removed */
-  if (default_workgroup != NULL)
-    smbc_setWorkgroup (smb_context, default_workgroup);
+  if (op_backend->default_workgroup != NULL)
+    smbc_setWorkgroup (smb_context, op_backend->default_workgroup);
 
 #ifndef DEPRECATED_SMBC_INTERFACE
   smb_context->flags = 0;
 #endif
 
+  /* Initial settings: 
+   *   - use Kerberos (always) 
+   *   - in case of no username specified, try anonymous login 
+   */
   smbc_setOptionUseKerberos (smb_context, 1);
-  smbc_setOptionFallbackAfterKerberos (smb_context, 1);  
-  //smbc_setOptionNoAutoAnonymousLogin (smb_context, 1);
-  
+  smbc_setOptionFallbackAfterKerberos (smb_context, op_backend->user != NULL);
+  smbc_setOptionNoAutoAnonymousLogin (smb_context, op_backend->user != NULL);
 
 #if 0
   smbc_setOptionDebugToStderr (smb_context, 1);
@@ -723,6 +915,101 @@ do_mount (GVfsBackend *backend,
   g_vfs_backend_set_mount_spec (backend, browse_mount_spec);
   g_mount_spec_unref (browse_mount_spec);
 
+  op_backend->mount_source = mount_source;
+  op_backend->mount_try = 0;
+  op_backend->password_save = G_PASSWORD_SAVE_NEVER;
+
+  smbc_opendir = smbc_getFunctionOpendir (smb_context);
+  smbc_closedir = smbc_getFunctionClosedir (smb_context);
+
+  uri = g_string_new ("smb://");
+
+  if (op_backend->server)
+    {
+      g_string_append_encoded (uri, op_backend->server, NULL, NULL);
+      g_string_append_c (uri, '/');
+    }
+
+  DEBUG ("do_mount - URI = %s\n", uri->str);
+
+  do
+    {
+      op_backend->mount_try_again = FALSE;
+      op_backend->mount_cancelled = FALSE;
+
+      DEBUG ("do_mount - try #%d \n", op_backend->mount_try);
+
+      dir = smbc_opendir (smb_context, uri->str);
+
+      DEBUG ("do_mount - [%s; %d] dir = %p, cancelled = %d, errno = [%d] '%s' \n", 
+             uri->str, op_backend->mount_try, dir, op_backend->mount_cancelled, 
+             errno, g_strerror (errno));
+
+      if (dir == NULL && 
+          (op_backend->mount_cancelled || (errno != EPERM && errno != EACCES)))
+        {
+	  DEBUG ("do_mount - (errno != EPERM && errno != EACCES), breaking\n");
+	  break;
+	}
+
+      if (dir != NULL)
+        {
+          /*  Let update_cache() do enumeration, check for the smbc_getdents() result */
+          res = update_cache (op_backend, dir);
+          smbc_closedir (smb_context, dir);
+          DEBUG ("do_mount - login successful, res = %d\n", res);
+          if (res)
+            break;
+        }
+	else {
+	  /*  Purge the cache, we need to have clean playground for next auth try  */
+	  purge_cached (smb_context);
+	}
+
+      /* The first round is Kerberos-only.  Only if this fails do we enable
+       * NTLMSSP fallback (turning off anonymous fallback, which we've
+       * already tried and failed with).
+       */
+      if (op_backend->mount_try == 0)
+        {
+          DEBUG ("do_mount - after anon, enabling NTLMSSP fallback\n");
+          smbc_setOptionFallbackAfterKerberos (op_backend->smb_context, 1);
+          smbc_setOptionNoAutoAnonymousLogin (op_backend->smb_context, 1);
+        }
+      op_backend->mount_try++;
+    }
+  while (op_backend->mount_try_again);
+
+  g_string_free (uri, TRUE);
+
+  op_backend->mount_source = NULL;
+
+  if (dir == NULL)
+    {
+      if (op_backend->mount_cancelled)
+        g_vfs_job_failed (G_VFS_JOB (job),
+                         G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED,
+                         _("Password dialog cancelled"));
+      else
+        /* TODO: Error from errno? */
+        g_vfs_job_failed (G_VFS_JOB (job),
+			  G_IO_ERROR, G_IO_ERROR_FAILED,
+			  /* translators: We tried to mount a windows (samba) share, but failed */
+			  _("Failed to retrieve share list from server"));
+
+      return;
+    }
+
+  g_vfs_keyring_save_password (op_backend->last_user,
+			       op_backend->server,
+			       op_backend->last_domain,
+			       "smb",
+			       NULL,
+			       NULL,
+			       0,
+			       op_backend->last_password,
+			       op_backend->password_save);
+
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
@@ -780,7 +1067,7 @@ run_mount_mountable (GVfsBackendSmbBrowse *backend,
   GError *error = NULL;
   GMountSpec *mount_spec;
 
-  g_mutex_lock (backend->entries_lock);
+  g_mutex_lock (&backend->entries_lock);
   
   entry = find_entry_unlocked (backend, filename);
 
@@ -803,7 +1090,7 @@ run_mount_mountable (GVfsBackendSmbBrowse *backend,
 			 G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
 			 _("File doesn't exist"));
       
-  g_mutex_unlock (backend->entries_lock);
+  g_mutex_unlock (&backend->entries_lock);
 
   if (error)
     {
@@ -822,7 +1109,7 @@ do_mount_mountable (GVfsBackend *backend,
 {
   GVfsBackendSmbBrowse *op_backend = G_VFS_BACKEND_SMB_BROWSE (backend);
 
-  update_cache (op_backend);
+  update_cache (op_backend, NULL);
 
   run_mount_mountable (op_backend,
 		       job,
@@ -878,7 +1165,7 @@ do_open_for_read (GVfsBackend *backend,
 {
   GVfsBackendSmbBrowse *op_backend = G_VFS_BACKEND_SMB_BROWSE (backend);
 
-  update_cache (op_backend);
+  update_cache (op_backend, NULL);
 
   run_open_for_read (op_backend, job, filename);
 }
@@ -948,6 +1235,7 @@ get_file_info_from_entry (GVfsBackendSmbBrowse *backend, BrowseEntry *entry, GFi
   g_file_info_set_edit_name (info, entry->name_utf8);
   g_file_info_set_attribute_string (info, "smb::comment", entry->comment);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_VIRTUAL, TRUE);
+  g_file_info_set_content_type (info, "inode/directory");
 
   icon = NULL;
   if (entry->smbc_type == SMBC_WORKGROUP)
@@ -1025,14 +1313,14 @@ run_query_info (GVfsBackendSmbBrowse *backend,
 {
   BrowseEntry *entry;
 
-  g_mutex_lock (backend->entries_lock);
+  g_mutex_lock (&backend->entries_lock);
   
   entry = find_entry_unlocked (backend, filename);
 
   if (entry)
     get_file_info_from_entry (backend, entry, info);
       
-  g_mutex_unlock (backend->entries_lock);
+  g_mutex_unlock (&backend->entries_lock);
 
   if (entry)
     g_vfs_job_succeeded (G_VFS_JOB (job));
@@ -1052,7 +1340,7 @@ do_query_info (GVfsBackend *backend,
 {
   GVfsBackendSmbBrowse *op_backend = G_VFS_BACKEND_SMB_BROWSE (backend);
 
-  update_cache (op_backend);
+  update_cache (op_backend, NULL);
 
   run_query_info (op_backend, job, filename, info, matcher);
 }
@@ -1067,7 +1355,6 @@ try_query_info (GVfsBackend *backend,
 		GFileAttributeMatcher *matcher)
 {
   GVfsBackendSmbBrowse *op_backend = G_VFS_BACKEND_SMB_BROWSE (backend);
-  const char *icon_name;
   GIcon *icon;
 
   if (is_root (filename))
@@ -1075,13 +1362,10 @@ try_query_info (GVfsBackend *backend,
       g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
       g_file_info_set_name (info, "/");
       g_file_info_set_display_name (info, g_vfs_backend_get_display_name (backend));
-      icon_name = g_vfs_backend_get_icon_name (backend);
-      if (icon_name)
-	{
-	  icon = g_themed_icon_new (icon_name);
-	  g_file_info_set_icon (info, icon);
-	  g_object_unref (icon);
-	}
+      g_file_info_set_content_type (info, "inode/directory");
+      icon = g_vfs_backend_get_icon (backend);
+      if (icon != NULL)
+        g_file_info_set_icon (info, icon);
       g_vfs_job_succeeded (G_VFS_JOB (job));
       
       return TRUE;
@@ -1121,7 +1405,7 @@ run_enumerate (GVfsBackendSmbBrowse *backend,
   g_vfs_job_succeeded (G_VFS_JOB (job));
 
   files = NULL;
-  g_mutex_lock (backend->entries_lock);
+  g_mutex_lock (&backend->entries_lock);
   for (l = backend->entries; l != NULL; l = l->next)
     {
       BrowseEntry *entry = l->data;
@@ -1131,7 +1415,7 @@ run_enumerate (GVfsBackendSmbBrowse *backend,
 
       files = g_list_prepend (files, info);
     }
-  g_mutex_unlock (backend->entries_lock);
+  g_mutex_unlock (&backend->entries_lock);
   
   files = g_list_reverse (files);
 
@@ -1150,8 +1434,8 @@ do_enumerate (GVfsBackend *backend,
 	      GFileQueryInfoFlags flags)
 {
   GVfsBackendSmbBrowse *op_backend = G_VFS_BACKEND_SMB_BROWSE (backend);
-  
-  update_cache (op_backend);
+
+  update_cache (op_backend, NULL);
 
   run_enumerate (op_backend, job, filename, matcher);
 }
@@ -1178,9 +1462,6 @@ g_vfs_backend_smb_browse_class_init (GVfsBackendSmbBrowseClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GVfsBackendClass *backend_class = G_VFS_BACKEND_CLASS (klass);
-#ifdef HAVE_GCONF
-  GConfClient *gclient;
-#endif
   
   gobject_class->finalize = g_vfs_backend_smb_browse_finalize;
 
@@ -1197,25 +1478,6 @@ g_vfs_backend_smb_browse_class_init (GVfsBackendSmbBrowseClass *klass)
   backend_class->try_query_info = try_query_info;
   backend_class->enumerate = do_enumerate;
   backend_class->try_enumerate = try_enumerate;
-
-#ifdef HAVE_GCONF
-  gclient = gconf_client_get_default ();
-  if (gclient)
-    {
-      char *workgroup;
-      
-      workgroup = gconf_client_get_string (gclient, 
-					   PATH_GCONF_GNOME_VFS_SMB_WORKGROUP, NULL);
-
-      if (workgroup && workgroup[0])
-	default_workgroup = workgroup;
-      else
-	g_free (workgroup);
-      
-      g_object_unref (gclient);
-    }
-#endif
-
 }
 
 void
